==> cosmic-lib-1.7.0.tgz: package/es5/parse-sep7.js <==
"use strict";
/**
 * Sep-0007 support
 * https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0007.md
 *
 * @private
 */

var _regeneratorRuntime = require("@babel/runtime/regenerator");

var _asyncToGenerator = require("@babel/runtime/helpers/asyncToGenerator");

var parseSep7 = exports;

var Buffer = require("@cosmic-plus/base/es5/buffer");

var StellarSdk = require("@cosmic-plus/base/es5/stellar-sdk");

var _require = require("@cosmic-plus/jsutils/es5/misc"),
    timeout = _require.timeout;

var check = require("./check");

var convert = require("./convert");

var decode = require("./decode");

var parse = require("./parse");
/**
 * Initialize cosmicLink using `sep7Request`.
 */


parseSep7.request = function (cosmicLink, sep7Request, options) {
  parse.page(cosmicLink, sep7Request);
  var query = convert.uriToQuery(cosmicLink, sep7Request);
  var sep7 = decodeURIComponent(query.substr(5));
  return parseSep7.link(cosmicLink, sep7, options);
};
/**
 * Initialize cosmicLink using `sep7`.
 */


parseSep7.link = function (cosmicLink, sep7) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  cosmicLink._sep7 = sep7;
  if (!options.network) options.network = "public";

  if (sep7.substr(12, 4) === "pay?") {
    cosmicLink.extra.type = "pay";
    return parseSep7.link.pay(cosmicLink, sep7, options);
  } else if (sep7.substr(12, 3) === "tx?") {
    cosmicLink.extra.type = "tx";
    return parseSep7.link.xdr(cosmicLink, sep7, options);
  } else {
    throw new Error("Invalid SEP-0007 link.");
  }
};
/**
 * Initialize cosmicLink using `sep7.xdr`.
 */


parseSep7.link.xdr = function (cosmicLink, sep7) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var query = convert.uriToQuery(cosmicLink, sep7);
  var params = query.substr(1).split("&");
  var xdr;
  params.forEach(function (entry) {
    var field = entry.replace(/=.*$/, "");
    var value = entry.substr(field.length + 1);

    switch (field) {
      case "xdr":
        xdr = decodeURIComponent(value);
        break;

      case "pubkey":
        check.address(cosmicLink, value);
        cosmicLink.extra.pubkey = value;
        break;

      default:
        parseSep7.link.common(cosmicLink, "xdr", field, value, options);
    }
  });
  if (!xdr) throw new Error("Missing XDR parameter");
  return {
    type: "xdr",
    value: xdr,
    options: options
  };
};
/**
 * Initialize cosmicLink using `sep7.pay`.
 */


parseSep7.link.pay = function (cosmicLink, sep7) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var query = convert.uriToQuery(cosmicLink, sep7);
  var params = query.substr(1).split("&");
  var odesc = {
    type: "payment"
  },
      asset = [],
      memo = [];
  var tdesc = {
    network: options.network,
    operations: [odesc] // Parse

  };
  params.forEach(function (entry) {
    var field = entry.replace(/=.*$/, "");
    var value = entry.substr(field.length + 1);

    switch (field) {
      case "destination":
      case "amount":
        odesc[field] = decode.field(cosmicLink, field, value);
        break;

      case "asset_code":
        asset[0] = value;
        break;

      case "asset_issuer":
        asset[1] = value;
        break;

      case "memo_type":
        memo[0] = value.split("_")[1].toLowerCase();
        break;

      case "memo":
        memo[1] = value;
        break;

      default:
        parseSep7.link.common(cosmicLink, "pay", field, value, options);
    }
  }); // Convert

  if (memo.length) {
    if (memo[0] === "hash" || memo[0] === "return") {
      var buffer = Buffer.from(decodeURIComponent(memo[1]), "base64");
      memo[1] = buffer.toString("hex");
    }

    tdesc.memo = decode.memo(cosmicLink, "".concat(memo[0], ":").concat(memo[1]));
  }

  if (asset.length) {
    odesc.asset = decode.asset(cosmicLink, "".concat(asset[0], ":").concat(asset[1]));
  }

  if (!odesc.destination) throw new Error("Missing parameter: destination");
  return {
    type: "tdesc",
    value: tdesc,
    options: options
  };
};

parseSep7.link.common = function (cosmicLink, mode, field, value, options) {
  switch (field) {
    case "network_passphrase":
      options.network = decode.network(cosmicLink, value);
      break;

    case "callback":
      value = decode.url(cosmicLink, value);

      if (value.substr(0, 4) !== "url:") {
        throw new Error("Invalid callback: " + value);
      }

      options.callback = value.substr(4);
      break;

    case "origin_domain":
      cosmicLink.extra.originDomain = parseSep7.checkSignature(cosmicLink, value);
      break;

    case "signature":
      cosmicLink.extra.signature = decodeURIComponent(value);
      break;

    case "msg":
      cosmicLink.extra.msg = decode.string(cosmicLink, value);
      break;

    default:
      throw new Error("Invalid SEP-0007 ".concat(mode, " field: ") + field);
  }
};

parseSep7.checkSignature =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(cosmicLink, domain) {
    var link, signature, toml, signingKey, keypair, payload;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            link = cosmicLink.sep7.replace(/&signature=.*/, ""); // Let parser parse signature.

            _context.next = 3;
            return timeout(1);

          case 3:
            signature = cosmicLink.extra.signature;

            if (signature) {
              _context.next = 6;
              break;
            }

            throw new Error("No signature attached for domain: ".concat(domain));

          case 6:
            _context.next = 8;
            return StellarSdk.StellarTomlResolver.resolve(domain);

          case 8:
            toml = _context.sent;
            signingKey = toml.URI_REQUEST_SIGNING_KEY;

            if (signingKey) {
              _context.next = 12;
              break;
            }

            throw new Error("Can't find signing key for domain: ".concat(domain));

          case 12:
            keypair = StellarSdk.Keypair.fromPublicKey(signingKey);
            payload = makePayload(link);

            if (!keypair.verify(payload, Buffer.from(signature, "base64"))) {
              _context.next = 18;
              break;
            }

            return _context.abrupt("return", domain);

          case 18:
            throw new Error("Invalid signature for domain: ".concat(domain));

          case 19:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

function makePayload(link) {
  return Buffer.concat([Buffer.alloc(35), Buffer.alloc(1, 4), Buffer.from("stellar.sep.7 - URI Scheme".concat(link))]);
}

==> cosmic-lib-1.8.0.tgz: package/es5/parse-sep7.js <==
"use strict";
/**
 * Sep-0007 support
 * https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0007.md
 *
 * @private
 */

var _regeneratorRuntime = require("@babel/runtime/regenerator");

var _asyncToGenerator = require("@babel/runtime/helpers/asyncToGenerator");

var parseSep7 = exports;

var Buffer = require("@cosmic-plus/base/es5/buffer");

var StellarSdk = require("@cosmic-plus/base/es5/stellar-sdk");

var _require = require("@cosmic-plus/jsutils/es5/misc"),
    timeout = _require.timeout;

var check = require("./check");

var convert = require("./convert");

var decode = require("./decode");

var parse = require("./parse");
/**
 * Initialize cosmicLink using `sep7Request`.
 */


parseSep7.request = function (cosmicLink, sep7Request, options) {
  parse.page(cosmicLink, sep7Request);
  var query = convert.uriToQuery(cosmicLink, sep7Request);
  var sep7 = decodeURIComponent(query.substr(5));
  return parseSep7.link(cosmicLink, sep7, options);
};
/**
 * Initialize cosmicLink using `sep7`.
 */


parseSep7.link = function (cosmicLink, sep7) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  cosmicLink._sep7 = sep7;
  if (!options.network) options.network = "public";

  if (sep7.substr(12, 4) === "pay?") {
    cosmicLink.extra.type = "pay";
    return parseSep7.link.pay(cosmicLink, sep7, options);
  } else if (sep7.substr(12, 3) === "tx?") {
    cosmicLink.extra.type = "tx";
    return parseSep7.link.xdr(cosmicLink, sep7, options);
  } else {
    throw new Error("Invalid SEP-0007 link.");
  }
};
/**
 * Initialize cosmicLink using `sep7.xdr`.
 */


parseSep7.link.xdr = function (cosmicLink, sep7) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var query = convert.uriToQuery(cosmicLink, sep7);
  var params = query.substr(1).split("&");
  var xdr;
  params.forEach(function (entry) {
    var field = entry.replace(/=.*$/, "");
    var value = entry.substr(field.length + 1);

    switch (field) {
      case "xdr":
        xdr = decodeURIComponent(value);
        break;

      case "pubkey":
        check.address(cosmicLink, value);
        cosmicLink.extra.pubkey = value;
        break;

      default:
        parseSep7.link.common(cosmicLink, "xdr", field, value, options);
    }
  });
  if (!xdr) throw new Error("Missing XDR parameter");
  return {
    type: "xdr",
    value: xdr,
    options: options
  };
};
/**
 * Initialize cosmicLink using `sep7.pay`.
 */


parseSep7.link.pay = function (cosmicLink, sep7) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var query = convert.uriToQuery(cosmicLink, sep7);
  var params = query.substr(1).split("&");
  var odesc = {
    type: "payment"
  },
      asset = {},
      memo = {};
  var tdesc = {
    network: options.network,
    operations: [odesc] // Parse

  };
  params.forEach(function (entry) {
    var field = entry.replace(/=.*$/, "");
    var value = entry.substr(field.length + 1);

    switch (field) {
      case "destination":
      case "amount":
        odesc[field] = decode.field(cosmicLink, field, value);
        break;

      case "asset_code":
        asset.code = decodeURIComponent(value);
        break;

      case "asset_issuer":
        asset.issuer = decodeURIComponent(value);
        break;

      case "memo_type":
        memo.type = value.split("_")[1].toLowerCase();
        break;

      case "memo":
        memo.value = decode.string(cosmicLink, value);
        break;

      default:
        parseSep7.link.common(cosmicLink, "pay", field, value, options);
    }
  }); // Convert

  if (memo.type || memo.value) {
    if (memo.type === "hash" || memo.type === "return") {
      memo.value = Buffer.from(memo.value, "base64").toString("hex");
    }

    tdesc.memo = memo;
  }

  if (asset.code || asset.issuer) odesc.asset = asset;
  if (!odesc.destination) throw new Error("Missing parameter: destination");
  return {
    type: "tdesc",
    value: tdesc,
    options: options
  };
};

parseSep7.link.common = function (cosmicLink, mode, field, value, options) {
  switch (field) {
    case "network_passphrase":
      options.network = decode.network(cosmicLink, value);
      break;

    case "callback":
      value = decode.url(cosmicLink, value);

      if (value.substr(0, 4) !== "url:") {
        throw new Error("Invalid callback: " + value);
      }

      options.callback = value.substr(4);
      break;

    case "origin_domain":
      cosmicLink.extra.originDomain = parseSep7.checkSignature(cosmicLink, value);
      break;

    case "signature":
      cosmicLink.extra.signature = decodeURIComponent(value);
      break;

    case "msg":
      cosmicLink.extra.msg = decode.string(cosmicLink, value);
      break;

    default:
      throw new Error("Invalid SEP-0007 ".concat(mode, " field: ") + field);
  }
};

parseSep7.checkSignature =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(cosmicLink, domain) {
    var link, signature, toml, signingKey, keypair, payload;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            link = cosmicLink.sep7.replace(/&signature=.*/, ""); // Let parser parse signature.

            _context.next = 3;
            return timeout(1);

          case 3:
            signature = cosmicLink.extra.signature;

            if (signature) {
              _context.next = 6;
              break;
            }

            throw new Error("No signature attached for domain: ".concat(domain));

          case 6:
            _context.next = 8;
            return StellarSdk.StellarTomlResolver.resolve(domain);

          case 8:
            toml = _context.sent;
            signingKey = toml.URI_REQUEST_SIGNING_KEY;

            if (signingKey) {
              _context.next = 12;
              break;
            }

            throw new Error("Can't find signing key for domain: ".concat(domain));

          case 12:
            keypair = StellarSdk.Keypair.fromPublicKey(signingKey);
            payload = makePayload(link);

            if (!keypair.verify(payload, Buffer.from(signature, "base64"))) {
              _context.next = 18;
              break;
            }

            return _context.abrupt("return", domain);

          case 18:
            throw new Error("Invalid signature for domain: ".concat(domain));

          case 19:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

function makePayload(link) {
  return Buffer.concat([Buffer.alloc(35), Buffer.alloc(1, 4), Buffer.from("stellar.sep.7 - URI Scheme".concat(link))]);
}
